94057c3d3e
PR Test (NPU) / check-changes (push) Has been cancelled
PR Test (NPU) / pr-gate (push) Has been cancelled
PR Test (NPU) / set-image-config (push) Has been cancelled
PR Test (NPU) / stage-b-test-1-npu-a2 (0) (push) Has been cancelled
PR Test (NPU) / stage-b-test-1-npu-a2 (1) (push) Has been cancelled
PR Test (NPU) / stage-b-test-2-npu-a2 (0) (push) Has been cancelled
PR Test (NPU) / stage-b-test-2-npu-a2 (1) (push) Has been cancelled
PR Test (NPU) / stage-b-test-4-npu-a3 (push) Has been cancelled
PR Test (NPU) / stage-b-test-16-npu-a3 (push) Has been cancelled
PR Test (NPU) / multimodal-gen-test-1-npu-a3 (push) Has been cancelled
PR Test (NPU) / multimodal-gen-test-2-npu-a3 (push) Has been cancelled
PR Test (Arm64) / pr-gate (push) Has been cancelled
PR Test (Arm64) / check-changes (push) Has been cancelled
PR Test (Arm64) / build-test (push) Has been cancelled
PR Test (sgl-router) / gate (push) Has been cancelled
PR Test (sgl-router) / tier-1 — lint (push) Has been cancelled
PR Test (sgl-router) / tier-2 — build + test (push) Has been cancelled
PR Test (sgl-router) / tier-3 — docker (placeholder) (push) Has been cancelled
PR Test (sgl-router) / tier-3 — k8s integration (push) Has been cancelled
PR Test (sgl-router) / tier-3 — e2e (push) Has been cancelled
PR Test (sgl-router) / finish (push) Has been cancelled
PR Test (NPU) / single-node-poc (map[name:qwen3_6_27b_w8a8_1p_in64k_out1k_50ms runner:linux-aarch64-a3-2 test_case:test/registered/ascend/performance/qwen3_6_27b/test_npu_qwen3_6_27b_w8a8_1p_in64k_out1k_50ms.py test_type:perf]) (push) Has been cancelled
PR Test (NPU) / pr-test-npu-finish (push) Has been cancelled
PR Test (Xeon) / pr-gate (push) Has been cancelled
PR Test (Xeon) / check-changes (push) Has been cancelled
PR Test (Xeon) / build-test (, xeon-gnr, base-b-test-cpu) (push) Has been cancelled
PR Test (XPU) / check-changes (push) Has been cancelled
PR Test (XPU) / pr-gate (push) Has been cancelled
PR Test (XPU) / stage-a-test-1-gpu-xpu (push) Has been cancelled
PR Test (XPU) / wait-for-stage-a (push) Has been cancelled
PR Test (XPU) / stage-b-test-1-gpu-xpu (push) Has been cancelled
PR Test (XPU) / finish (push) Has been cancelled
CI Model Inventory / build-inventory (push) Has been cancelled
Lint / lint (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark Compilation Check (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark - Manual Policy (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark - Request Processing (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark Summary (push) Has been cancelled
PR Test (SMG) / build-wheel (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on windows (x86_64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on macos (x86_64 - auto) (push) Has been cancelled
PR Test (SMG) / python-unit-tests (push) Has been cancelled
PR Test (SMG) / unit-tests (push) Has been cancelled
PR Test (SMG) / benchmarks (push) Has been cancelled
PR Test (SMG) / chat-completions (push) Has been cancelled
PR Test (SMG) / chat-completions-4gpu (push) Has been cancelled
PR Test (SMG) / e2e (push) Has been cancelled
PR Test (SMG) / docker-build-test (push) Has been cancelled
PR Test (SMG) / k8s-integration (push) Has been cancelled
PR Test (SMG) / finish (push) Has been cancelled
PR Test (SMG) / summarize-benchmarks (push) Has been cancelled
Release SGLang Model Gateway Docker Image / publish (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on macos (aarch64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (aarch64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (x86_64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (aarch64 - musllinux_1_1) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (x86_64 - musllinux_1_1) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / Build SDist (push) Has been cancelled
Release SGLang Model Gateway to PyPI / Upload to PyPI (push) Has been cancelled
Release SGLang Kernels / build-cu129-matrix (aarch64, 12.9, 3.10, arm-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / build-cu129-matrix (x86_64, 12.9, 3.10, x64-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / release-cu129 (push) Has been cancelled
Release SGLang Kernels / build-cu130-matrix (aarch64, 13.0, 3.10, arm-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / build-cu130-matrix (x86_64, 13.0, 3.10, x64-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / release-cu130 (push) Has been cancelled
Release SGLang Kernels / build-rocm-matrix (3.10, 700) (push) Has been cancelled
Release SGLang Kernels / build-rocm-matrix (3.10, 720) (push) Has been cancelled
Release SGLang Kernels / release-rocm700 (push) Has been cancelled
Release SGLang Kernels / release-rocm720 (push) Has been cancelled
Release SGLang Kernels / build-musa43 (43, 3.10) (push) Has been cancelled
Release SGLang Kernels / release-musa43 (push) Has been cancelled
102 lines
4.1 KiB
Plaintext
102 lines
4.1 KiB
Plaintext
#include <sgl_kernel/tensor.h> // For TensorMatcher, SymbolicSize, SymbolicDevice
|
|
#include <sgl_kernel/utils.h> // For div_ceil, RuntimeCheck
|
|
|
|
#include <sgl_kernel/utils.cuh> // For LaunchKernel
|
|
#include <sgl_kernel/vec.cuh>
|
|
|
|
#include <dlpack/dlpack.h>
|
|
#include <tvm/ffi/container/tensor.h>
|
|
|
|
#include <cstddef>
|
|
#include <cstdint>
|
|
|
|
namespace {
|
|
|
|
constexpr size_t kBlockSize = 256;
|
|
constexpr size_t kVectorizedMinElements = 1 << 20;
|
|
constexpr size_t kVectorBytes = device::kMaxVecBytes;
|
|
static_assert(kVectorBytes % sizeof(int32_t) == 0, "Vector byte width must contain whole int32_t elements");
|
|
constexpr size_t kElementsPerVector = kVectorBytes / sizeof(int32_t);
|
|
|
|
template <typename Vector>
|
|
bool is_aligned_for_vector(const int32_t* ptr) {
|
|
return reinterpret_cast<uintptr_t>(ptr) % alignof(Vector) == 0;
|
|
}
|
|
|
|
template <int32_t kConstant>
|
|
__global__ void add_constant_kernel(int32_t* dst, const int32_t* src, size_t length) {
|
|
size_t idx = blockIdx.x * blockDim.x + threadIdx.x;
|
|
if (idx < length) {
|
|
dst[idx] = src[idx] + kConstant;
|
|
}
|
|
}
|
|
|
|
template <int32_t kConstant, size_t kElementsPerVector>
|
|
__global__ void add_constant_vectorized_kernel(int32_t* dst, const int32_t* src, size_t length) {
|
|
using Vector = device::AlignedVector<int32_t, kElementsPerVector>;
|
|
|
|
const size_t work_idx = blockIdx.x * blockDim.x + threadIdx.x;
|
|
const size_t vector_count = length / kElementsPerVector;
|
|
const size_t tail_start = vector_count * kElementsPerVector;
|
|
|
|
if (work_idx < vector_count) {
|
|
auto values = device::load_as<Vector>(src, work_idx);
|
|
#pragma unroll
|
|
for (size_t i = 0; i < kElementsPerVector; ++i) {
|
|
values[i] += kConstant;
|
|
}
|
|
device::store_as<Vector>(dst, values, work_idx);
|
|
} else {
|
|
const size_t tail_idx = tail_start + work_idx - vector_count;
|
|
if (tail_idx < length) {
|
|
dst[tail_idx] = src[tail_idx] + kConstant;
|
|
}
|
|
}
|
|
}
|
|
|
|
// You can also use struct with static method as an alternative
|
|
template <int32_t kConstant>
|
|
void add_constant(tvm::ffi::TensorView dst, tvm::ffi::TensorView src) {
|
|
using namespace host;
|
|
|
|
// 1. Validate input tensors
|
|
SymbolicSize N = {"num_elements"};
|
|
SymbolicDevice device_;
|
|
TensorMatcher({N}) // 1D tensor, must be contiguous
|
|
.with_dtype<int32_t>() // must be int32
|
|
.with_device<kDLGPU>(device_) // must be on GPU device (CUDA or ROCm)
|
|
.verify(dst) // check tensor dst
|
|
.verify(src); // check tensor src
|
|
|
|
// 2. Extract required parameters, prepare for kernel launch
|
|
const size_t num_elements = N.unwrap();
|
|
const DLDevice device = device_.unwrap();
|
|
[[maybe_unused]] // optional, can be omitted
|
|
const size_t dynamic_smem = 0;
|
|
[[maybe_unused]] // optional, LaunchKernel can auto determine stream from device
|
|
const cudaStream_t stream = LaunchKernel::resolve_device(device);
|
|
// some extra runtime checks using host::RuntimeCheck
|
|
RuntimeCheck(num_elements > 0, "We only support non-empty tensors, got num_elements = ", num_elements);
|
|
|
|
const auto* src_ptr = static_cast<const int32_t*>(src.data_ptr());
|
|
auto* dst_ptr = static_cast<int32_t*>(dst.data_ptr());
|
|
using Vector = device::AlignedVector<int32_t, kElementsPerVector>;
|
|
const bool is_vector_aligned = is_aligned_for_vector<Vector>(src_ptr) && is_aligned_for_vector<Vector>(dst_ptr);
|
|
|
|
// 3. Launch the kernel. Error code will be automatically checked.
|
|
if (num_elements >= kVectorizedMinElements && is_vector_aligned) {
|
|
const size_t vector_count = num_elements / kElementsPerVector;
|
|
const size_t tail_count = num_elements - vector_count * kElementsPerVector;
|
|
const size_t work_items = vector_count + tail_count;
|
|
const size_t grid_size = div_ceil(work_items, kBlockSize);
|
|
LaunchKernel(grid_size, kBlockSize, device /*, dynamic_smem*/)(
|
|
add_constant_vectorized_kernel<kConstant, kElementsPerVector>, dst_ptr, src_ptr, num_elements);
|
|
} else {
|
|
const size_t grid_size = div_ceil(num_elements, kBlockSize);
|
|
LaunchKernel(grid_size, kBlockSize, device /*, dynamic_smem*/)(
|
|
add_constant_kernel<kConstant>, dst_ptr, src_ptr, num_elements);
|
|
}
|
|
}
|
|
|
|
} // namespace
|